from .ggs_stuff import keeping
from .just_funcs import fakescanning, getserver
from .getconfig import getconfig
from .pstorm import ggs_login
__all__ = ["ggs_login", "fakescanning", "getconfig", "keeping", "getserver"]
__version__ = "1.2.0"
